import time
import urllib
import xml.etree.ElementTree as ET

import requests

import utils


ALARM_DURATION = 60


class CameraXMLResponse(object):
    ''' A dictionary-like container which parses the XML response to a CGI request '''

    RESULT_MSG = { 0: "Success",
                  -1: "CGI request string format error",
                  -2: "Username or password error",
                  -3: "Access denied",
                  -4: "CGI execute failure",
                  -5: "Timeout"
                  }

    def __init__(self, response):
        self._xml = response.text
        self._xml_root = ET.fromstring(self._xml)

        result = self._xml_root.find('result')
        self._result_value = int(result.text)
        self._xml_root.remove(result)

    def __nonzero__(self):
        return self._result_value == 0
    
    def __str__(self):
        return self._xml

    def __getitem__(self, key):
        return int(self._xml_root.find(key).text)

    def __iter__(self):
        return (element.tag for element in self._xml_root)

    keys = __iter__

    def items(self):
        return ((element.tag, int(element.text)) for element in self._xml_root)

    def values(self):
        return (int(element.text) for element in self._xml_root)

    @property
    def status(self):
        return self._result_value

    @property
    def message(self):
        return self.RESULT_MSG[self._result_value]


class SetConfigCommand(object):
    def __init__(self, camera, cmd):
        self.camera = camera
        self.cmd = cmd
        
        get_cmd = self.cmd.replace("set", "get")
        self._config = dict(self.camera.send_command(get_cmd).items())

    def __setitem__(self, key, value):
        self._config[key] = value

    def send(self):
        return self.camera.send_command(self.cmd, self._config)


class Camera(object):
    def __init__(self, host, port, user, password):
        #self._url_fmt = "http://{0}:{1}/decoder_control.cgi?command={{0}}&user={2}&pwd={3}"
        self._url_fmt = "http://{0}:{1}/decoder_control.cgi?command={{0}}&user={2}&pwd={3}".format(host,
                                                                                                port,
                                                                                                user,
                                                                                                password)
                                                                                              
        self._url_cameracontrol = "http://{0}:{1}/camera_control.cgi?{{0}}&user={2}&pwd={3}".format(host,
                                                                                                port,
                                                                                                user,
                                                                                                password)
                                                                                                
        #param=5&value=0

        #self._video_url = "http://{0}:{1}/videostream.cgi?command={{0}}&user={2}&pwd={3}".format(user, password, host, port)
        self._video_url = "http://"+str(host)+":"+str(port)+"/videostream.cgi?user="+str(user)+"&pwd="+str(password)

    @property
    def video_url(self):
        return self._video_url

    def send_command(self, cmd, params=None, data=False):
        print cmd
        if 'param=5' in cmd: url = self._url_cameracontrol.format(cmd)
        else: url = self._url_fmt.format(cmd)
        if params is not None:
            url += "&" + urllib.urlencode(params)

        utils.log_verbose(url)
        print url
        try:
            response = requests.get(url)
            print response.text
        except (requests.RequestException) as e:
            utils.log_error(str(e))
            return False
        else:
            if not response:
                return False
            elif data:
                print "sin e data"
                #return response.text
                return True
            else:
                print "entrouaqui"
                utils.log_verbose(response)
                xml_resp = CameraXMLResponse(response)
                utils.log_verbose(xml_resp)
                if not xml_resp:
                    utils.log_error(xml_resp.message)
                return xml_resp

    def test(self):
        #response = self.send_command("getDevState")
        #if response:
        #    msg = response.message
        #else:
        #    msg = "Error connecting to camera."
        #return bool(response), msg
        return True, 'yap'
    
    def move(self, direction):
        print "a direccao e",direction
        #print "ptzMove" + direction.capitalize()
        if direction == 'up': return self.send_command("0&onestep=1",data=True)
        elif direction == 'down': return self.send_command("2&onestep=1",data=True)
        elif direction == 'left': return self.send_command("4&onestep=1",data=True)
        elif direction == 'right': return self.send_command("6&onestep=1",data=True)
        #self.send_command("ptzMove" + direction.capitalize())
        #time.sleep(0.5)
        #self.send_command("ptzStopRun")
        #

    def get_mirror_and_flip(self):
        return self.send_command("getMirrorAndFlipSetting").values()

    def toggle_mirror_flip(self, action, enable):
        if action == 'flip':
            if int(enable) == 0: self.send_command('param=5&value=0')
            else: self.send_command('param=5&value=1')
        elif action == 'mirror':
            if int(enable) == 0: self.send_command('param=5&value=0')
            else: self.send_command('param=5&value=2')            
        else:
            print action.lower() + "Video", {"is" + action.capitalize(): int(enable)}
            return self.send_command(action.lower() + "Video", {"is" + action.capitalize(): int(enable)})

    def set_ir_on(self):
        return self.send_command('openInfraLed')

    def set_ir_off(self):
        return self.send_command('closeInfraLed')

    def get_motion_detect_config(self):
        return self.send_command('getMotionDetectConfig')

    def get_sound_detect_config(self):
        return self.send_command('getAudioAlarmConfig')
    
    def get_device_state(self):
        return self.send_command('getDevState')

    def get_snapshot_config(self):
        return self.send_command('getSnapConfig')

    def set_motion_detect_config(self):
        return SetConfigCommand(self, 'setMotionDetectConfig')
    
    def set_sound_detect_config(self):
        return SetConfigCommand(self, 'setAudioAlarmConfig')
    
    def set_snapshot_config(self):
        return SetConfigCommand(self, 'setSnapConfig')
    
    def get_snapshot(self):
        return self.send_command('snapPicture2', data=True)
        
        


